#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using OSU.DRSS.ManagedObjects;


namespace OSU.DRSS.Platform
{

	public delegate void InterceptorRegisteredEventDelegate(object objSource, InterceptorRegisteredEventArgs ireArgs);


	[Serializable]
	public class InterceptorRegisteredEventArgs : EventArgs
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------
		// The chain ID corresponding to the chain into which the interceptor
		// was inserted
		private Guid guiChainID;

		// The chain index at which the interceptor was inserted
		private int intIndex;
		
		// The ManagedObjectInformation corresponding to the interceptor that was
		// registered
		private ManagedObjectInformation moiInterceptorInformation;
		//-----------------------------------------------------------------
		#endregion

		
		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public InterceptorRegisteredEventArgs(Guid guiChainID, int intIndex, ManagedObjectInformation moiInterceptorInformation)
		{
			this.guiChainID = guiChainID;
			this.intIndex = intIndex;
			this.moiInterceptorInformation = moiInterceptorInformation;
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Properties
		//-----------------------------------------------------------------		
		// Properties
		//-----------------------------------------------------------------	
		public Guid ChainID
		{
			get
			{	return(guiChainID);   }
		}

		public int Index
		{
			get
			{	return(intIndex);   }
		}

		public ManagedObjectInformation InterceptorInformation
		{
			get
			{	return(moiInterceptorInformation);   }
		}
		//-----------------------------------------------------------------	
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------		
		public override string ToString()
		{	
			string strResult = "Interceptor Registered: [" + moiInterceptorInformation + "]; " +
							   "Chain: [" + guiChainID + "]; Index: [" + intIndex + "]";
			return(strResult);   
		}
		//-----------------------------------------------------------------		
		#endregion
	}

}
