#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using System.Threading;
using System.Collections;
using OSU.DRSS.ManagedObjects;


namespace OSU.DRSS.Platform.Messaging.Interception
{

	public abstract class InterceptorChain : ManagedObjectBase, IEnumerable
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------		
		// The underlying array of interceptors contained in the chain
		protected ArrayList alInterceptors;

		// The object used to provide readers/writers synchronization 
		protected ReaderWriterLock rwlLock;
		//-----------------------------------------------------------------		
		#endregion


		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public InterceptorChain(string strName, string strDescription, object objSpecification)
			: base(strName, strDescription, objSpecification)
		{	
			this.alInterceptors = new ArrayList();
			this.rwlLock = new ReaderWriterLock();
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Properties
		//-----------------------------------------------------------------		
		// Properties
		//-----------------------------------------------------------------	
		public int InterceptorCount
		{
			get
			{
				//---
				// Reader method
				rwlLock.AcquireReaderLock(0);
				int intValue = alInterceptors.Count;
				rwlLock.ReleaseReaderLock();
				return(intValue);
				//---
			}
		}
		//-----------------------------------------------------------------	
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------		
		public void RegisterInterceptor(int intIndex, object objInterceptor)
		{
			//---
			// Writer method
			rwlLock.AcquireWriterLock(0);			
			alInterceptors.Insert(intIndex, objInterceptor);
			rwlLock.ReleaseWriterLock();
			//---
		}

		public void UnregisterInterceptor(int intIndex)
		{
			//---
			// Writer method
			rwlLock.AcquireWriterLock(0);			
			alInterceptors.RemoveAt(intIndex);
			rwlLock.ReleaseWriterLock();
			//---
		}

		public IEnumerator GetEnumerator()
		{	return MOIEnumeratorFactory.GetMOIEnumerator(alInterceptors);   }
		//-----------------------------------------------------------------		
		#endregion
	}
	
}
