#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using OSU.DRSS.Platform.Messaging.Transport;


namespace OSU.DRSS.Platform.Messaging.Interception
{
	
	public class ReceiveInterceptorChain : InterceptorChain, MessageReceiver
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------		
		// The router used to deliver messages to the various MessageReceiver sinks
		private LocalDeliveryRouter ldrLocalRouter;
		//-----------------------------------------------------------------		
		#endregion


		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public ReceiveInterceptorChain(string strName, string strDescription, object objSpecification)
			: base(strName, strDescription, objSpecification)
		{	this.ldrLocalRouter = new LocalDeliveryRouter();   }
		//-----------------------------------------------------------------		
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------		
		public void RegisterSink(Guid guiObjectID, MessageReceiver mrReceiver)
		{	ldrLocalRouter.RegisterMessageReceiver(guiObjectID, mrReceiver);   }

		public void UnregisterSink(Guid guiObjectID)
		{	ldrLocalRouter.UnregisterMessageReceiver(guiObjectID);   }

		public void ReceiveMessage(Message msg)
		{
			//-----
			// Reader method
			rwlLock.AcquireReaderLock(0);
			
			// Verify that appropriate interceptors exist to handle the content contained in msg
			bool bolContentProvidersRecognized = ContentProvidersRecognized(msg);
			if(bolContentProvidersRecognized)
			{
				//---
				// Pass the message to each interceptor in the chain.  If any interceptor
				// sets the message to null, stop processing the message
				for(int intIndex = 0; intIndex < alInterceptors.Count; intIndex++)
				{
					((ReceiveInterceptor)alInterceptors[intIndex]).InterceptMessage(ref msg);
					if(msg == null)
					{	break;   }
				}
				//---
			}

			rwlLock.ReleaseReaderLock();
			//-----

			// If the message content can not be recognized by the chain, throw an exception
			if(!bolContentProvidersRecognized)
			{	throw new UnrecognizedMessageContentException(msg);   }

			// If it was not set to null, deliver the message to the appropriate sink
			if(msg != null)
			{	ldrLocalRouter.ReceiveMessage(msg);   }
		}

		private bool ContentProvidersRecognized(Message msg)
		{
			// Check whether appropriate interceptors exist to handle the content contained in msg
			int intChainIndex = 0;
			int intLayerCount = 0;
			foreach(MessageContentLayer mclContent in msg.MessageContent)
			{
				// Note that the last content layer of the message will be forwarded by the local delivery router
				// to a destination sink.  The sink is responsible for verifying that the content of the final
				// layer is recognized
				intLayerCount++;
				if(intLayerCount != msg.MessageContent.Count)
				{
					// Skip interceptors that don't rely on a particular content type
					while((intChainIndex < alInterceptors.Count) &&
						  (((ReceiveInterceptor)alInterceptors[intChainIndex]).ContentConsumerID.Equals(Guid.Empty)))
					{   intChainIndex++;   }

					// Check whether the current content layer will be recognized by the appropriate interceptor
					if((intChainIndex == alInterceptors.Count) || 
					   (!((ReceiveInterceptor)alInterceptors[intChainIndex]).ContentConsumerID.Equals(mclContent.ContentProviderID)))
					{	return(false);   }
				}
			}
			
			return(true);
		}
		//-----------------------------------------------------------------		
		#endregion
	}

}
