#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;


namespace OSU.DRSS.Platform.Messaging.Interception
{

	public class SendInterceptorChain : InterceptorChain, MessageSender
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------		
		// The MessageSender sink into which all messages, unless canceled, are eventually forwarded
		private MessageSender msSink;
		//-----------------------------------------------------------------		
		#endregion


		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public SendInterceptorChain(MessageSender msSink, string strName, string strDescription, object objSpecification)
			: base(strName, strDescription, objSpecification)
		{	this.msSink = msSink;   }
		//-----------------------------------------------------------------		
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------		
		public void SendMessage(int intThreadID, Message msg)
		{	
			//-----
			// Reader method
			rwlLock.AcquireReaderLock(0);
			
			// Pass the message to each interceptor in the chain.  If any interceptor
			// sets the message to null, stop processing the message
			for(int intIndex = 0; intIndex < alInterceptors.Count; intIndex++)
			{
				((SendInterceptor)alInterceptors[intIndex]).InterceptMessage(intThreadID, ref msg);
				if(msg == null)
				{	break;   }
			}

			rwlLock.ReleaseReaderLock();
			//-----

			// If it was not set to null, forward the message to the MessageSender sink
			if(msg != null)
			{	msSink.SendMessage(intThreadID, msg);   }
		}
		//-----------------------------------------------------------------		
		#endregion
	}

}
