#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using System.Collections;


namespace OSU.DRSS.Platform.Messaging
{

	[Serializable]	
	public class Message
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------
		// The source (hosted object) address
		private PlatformHostedObjectAddress phoSourceAddress;

		// The destination (hosted object) address 
		private PlatformHostedObjectAddress phoDestinationAddress;
		
		// The content of the message: a stack of ContentLayer objects
		private Stack stkMessageContent;
		//-----------------------------------------------------------------		
		#endregion

		
		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public Message(PlatformHostedObjectAddress phoSourceAddress, PlatformHostedObjectAddress phoDestinationAddress)
		{
			this.phoSourceAddress = phoSourceAddress;
			this.phoDestinationAddress = phoDestinationAddress;
			this.stkMessageContent = new Stack();
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Properties
		//-----------------------------------------------------------------		
		// Properties
		//-----------------------------------------------------------------	
		public PlatformHostedObjectAddress SourceAddress
		{
			get
			{	return(phoSourceAddress);   }
		}

		public PlatformHostedObjectAddress DestinationAddress
		{
			get
			{	return(phoDestinationAddress);   }
		}

		public Stack MessageContent
		{
			get
			{	return(stkMessageContent);   }
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------		
		public override string ToString()
		{	
			string strResult = "Source Address: [" + phoSourceAddress + "] ; Destination Address: [" + phoDestinationAddress + "] ; ";
			
			//---
			// Include the stack entries in the returned string
			strResult += "Content: [";
			
			IEnumerator enmEntries = stkMessageContent.GetEnumerator();
			if(enmEntries.MoveNext())
			{
				strResult += enmEntries.Current;
				
				while(enmEntries.MoveNext())
				{	strResult += ", " + enmEntries.Current;   }
			}

			strResult += "]";
			//---
			
			return(strResult);
		}
		//-----------------------------------------------------------------		
		#endregion
	}

}
