#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using System.Collections;
using System.Threading;


namespace OSU.DRSS.Platform.Messaging.Transport
{

	public class LocalDeliveryRouter : MessageReceiver
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------		
		// The routing table which maps GUID objects to MessageReceiver objects
		private Hashtable htRoutingTable;		

		// The object used to provide readers/writers synchronization 
		private ReaderWriterLock rwlLock;
		//-----------------------------------------------------------------		
		#endregion


		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public LocalDeliveryRouter()
		{ 
			this.htRoutingTable = new Hashtable();
			this.rwlLock = new ReaderWriterLock();
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------		
		public void ReceiveMessage(Message msg)
		{
			//---
			// Reader method
			rwlLock.AcquireReaderLock(0);

			// Attempt to deliver the message to a registered MessageReceiver
			bool bolDelivered = false;
			if(htRoutingTable.Contains(msg.DestinationAddress.ObjectID))
			{	
				MessageReceiver mrReceiver = (MessageReceiver) htRoutingTable[msg.DestinationAddress.ObjectID];
				mrReceiver.ReceiveMessage(msg);
				bolDelivered = true;
			}
			
			rwlLock.ReleaseReaderLock();
			//---

			if(!bolDelivered)
			{	throw new MessageUndeliverableException(msg);   }
		}

		public void RegisterMessageReceiver(Guid guiObjectID, MessageReceiver mrReceiver)
		{
			//---
			// Writer method
			//
			// Hashtable objects can safely support one writer and multiple readers concurrently.
			// (No additional synchronization code is required here.)
			//
			htRoutingTable.Add(guiObjectID, mrReceiver);
			//---
		}

		public void UnregisterMessageReceiver(Guid guiObjectID)
		{	
			//---
			// Writer method
			rwlLock.AcquireWriterLock(0);

			htRoutingTable.Remove(guiObjectID); 

			rwlLock.ReleaseWriterLock();
			//---
		}
		//-----------------------------------------------------------------		
		#endregion
	}

}
