#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;


namespace OSU.DRSS.Platform.MethodInvocation
{
	
	[Serializable]
	public class MethodCallInformation
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------		
		// The GUID used to uniquely identify an invocation.  This is used to
		// match method results to the corresponding method call
		private Guid guiInvocationID;
		
		// The name of the method to be invoked
		private string strMethodName;

		// The parameter types expected by the method to be invoked
		private Type[] typParameterTypes;

		// The arguments passed to the method
		private object[] objArguments;
		//-----------------------------------------------------------------		
		#endregion


		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public MethodCallInformation(string strMethodName, Type[] typParameterTypes, object[] objArguments)
		{
			this.guiInvocationID = Guid.NewGuid();
			this.strMethodName = strMethodName;
			this.typParameterTypes = typParameterTypes;
			this.objArguments = objArguments;
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Properties
		//-----------------------------------------------------------------		
		// Properties
		//-----------------------------------------------------------------		
		public Guid InvocationID
		{
			get
			{	return(guiInvocationID);   }
		}	
		
		public string MethodName
		{
			get
			{	return(strMethodName);   }
		}

		public Type[] ParameterTypes
		{
			get
			{	return(typParameterTypes);   }
		}

		public object[] Arguments
		{
			get
			{	return(objArguments);   }
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------		
		public override string ToString()
		{
			string strResult = "Method Call: [" + strMethodName + "(";

			// Include the arguments and their respective types in the string returned
			for(int intIndex = 0; intIndex < objArguments.Length; intIndex++)
			{
				strResult += objArguments[intIndex];
				if(objArguments[intIndex] != null)
				{	strResult += " : " + objArguments[intIndex].GetType();   }
				
				if(intIndex != (objArguments.Length-1))
				{	strResult += ", ";   }
			}
			strResult += ")]; Invocation ID: [" + guiInvocationID + "]";

			return(strResult);
		}
		//-----------------------------------------------------------------		
		#endregion
	}

}
