#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;


namespace OSU.DRSS.Platform.MethodInvocation
{
	
	[Serializable]
	public class MethodResultInformation
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------		
		// The GUID used to uniquely identify an invocation.  This is used to
		// match method results to the corresponding method call
		private Guid guiInvocationID;
		
		// The post-invocation values of the method arguments
		private object[] objArguments;
		
		// A flag used to indicate whether the result of the method invocation was an exception
		private bool bolIsException;

		// The result of the method invocation
		private object objResult;
		//-----------------------------------------------------------------		
		#endregion
		

		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public MethodResultInformation(Guid guiInvocationID, object[] objArguments, bool bolIsException, object objResult)
		{
			this.guiInvocationID = guiInvocationID;
			this.objArguments = objArguments;
			this.bolIsException = bolIsException;
			this.objResult = objResult;
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Properties
		//-----------------------------------------------------------------		
		// Properties
		//-----------------------------------------------------------------		
		public Guid InvocationID
		{
			get
			{	return(guiInvocationID);   }
		}
		
		public object[] Arguments
		{
			get
			{	return(objArguments);   }
		}
		
		public bool IsException
		{
			get
			{	return(bolIsException);   }
		}

		public object Result
		{
			get
			{	return(objResult);   }
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------		
		public override string ToString()
		{
			string strResult = "Arguments: [";

			// Include the post-invocation values of the method arguments (as well as their respective types) 
			// in the string returned
			for(int intIndex = 0; intIndex < objArguments.Length; intIndex++)
			{
				strResult += objArguments[intIndex];
				if(objArguments[intIndex] != null)
				{	strResult += " : " + objArguments[intIndex].GetType();   }

				if(intIndex != (objArguments.Length-1))
				{	strResult += ", ";   }
			}
			strResult += "]; ";

			// Include the return value / exception, as well as the corresponding type in the
			// string returned
			if(bolIsException)
			{	strResult += "Exception: [" + ((Exception)objResult).Message + " : " + objResult.GetType() + "]; ";   }
			else
			{	
				strResult += "Return Value: [" + objResult;
				if(objResult != null)
				{	strResult += " : " + objResult.GetType();   }
				strResult += "]; ";   
			}

			strResult += "Invocation ID: [" + guiInvocationID + "]";
			return(strResult);
		}
		//-----------------------------------------------------------------		
		#endregion
	}

}
