#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason Hallstrom <hallstro@cis.ohio-state.edu>
// Bill Leal <leal@cis.ohio-state.edu>
//
// This file is part of DRSS.
//
// DRSS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DRSS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DRSS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using System.Threading;
using System.Reflection;
using OSU.DRSS.Platform.Messaging;
using OSU.DRSS.ManagedObjects;
using OSU.DRSS.Util;



namespace OSU.DRSS.Platform.MethodInvocation
{
	
	public class ObjectSkeleton : ManagedObjectBase, MessageReceiver
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------		
		// The target object for all method invocations
		private object objInvocationTarget;

		// The object used to send invocation results back to a client (stub)
		private MessageSender msSender;
		//-----------------------------------------------------------------		
		#endregion


		#region Constructors
		//-----------------------------------------------------------------		
		// Constructors
		//-----------------------------------------------------------------		
		public ObjectSkeleton(Guid guiObjectID, object objInvocationTarget, MessageSender msSender, string strName, string strDescription, object objSpecification)
			: base(guiObjectID, strName, strDescription, objSpecification)
		{	
			this.objInvocationTarget = objInvocationTarget;   
			this.msSender = msSender;
		}
		//-----------------------------------------------------------------		
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------
		public void ReceiveMessage(Message msg)
		{
			//---
			// Note that at this point there should be exactly one content layer on the message
			// content stack, containing the appropriate MethodCallInformation object.  The content 
			// provider of this message will always be the corresponding ObjectStub instance; no 
			// additional content provider/consumer verification is required
			//---
			
			try
			{
				//---
				// Retrieve the MethodInfo corresponding to the method to be invoked
				MethodCallInformation mciCall = (MethodCallInformation) ((MessageContentLayer)msg.MessageContent.Pop()).LayerContent;
				MethodInfo miMethod = objInvocationTarget.GetType().GetMethod(mciCall.MethodName, mciCall.ParameterTypes);
				//---
			
				//---
				// Invoke the method on the target object, and capture the results of the invocation.
				bool bolExceptionOccured = false;
				object objResult = null;
				try
				{	objResult = miMethod.Invoke(objInvocationTarget, mciCall.Arguments);   }
				catch(Exception exc)
				{	
					objResult = exc;   
					bolExceptionOccured = true;
				}
				//---
				
				//---		
				// Send the invocation results to the appropriate ObjectStub
				MethodResultInformation mriResult = new MethodResultInformation(mciCall.InvocationID, mciCall.Arguments, bolExceptionOccured, objResult);
				Message msgResult = new Message(msg.DestinationAddress, msg.SourceAddress);
				msgResult.MessageContent.Push(new MessageContentLayer(mriResult, Guid.Empty));
				msSender.SendMessage(Thread.CurrentThread.GetHashCode(), msgResult);
				//---		
			}
			catch(Exception exc)
			{
				// Something went wrong while processing the call -- generate the appropriate event
				if(CallProcessingExceptionEvent != null)
				{	CallProcessingExceptionEvent(this, new ExceptionEventArgs(exc));   }
			}
		}
		//-----------------------------------------------------------------
		#endregion


		#region Events
		//-----------------------------------------------------------------
		// Events
		//-----------------------------------------------------------------
		// This event is generated whenever an exception is raised while processing a method call
		public event ExceptionEventDelegate CallProcessingExceptionEvent;
		//-----------------------------------------------------------------
		#endregion
	}

}
